DISPLAYED_ALGO_COUNT <- 5 # nolint because lintr wants snake_case
DISPLAYED_REGISTRARS_COUNT <- 25 # nolint because lintr wants snake_case
past_year <- year(Sys.Date()) - 1

api_data <- fromJSON(api_endpoint("/fred_dnssec_domains_by_algorithm", zone = "eq.cz", order = "ts.asc"))

df_preprocessed <- api_data %>%
  mutate(
    Date = as.Date(ts),
    algorithm = as.factor(algorithm),
    Year = year(Date)
  )

get_latest_per_year <- function(data, year) {
  data %>%
    filter(Year == year) %>%
    filter(ts == max(ts))
}

past_years_to_include <- seq(2020, past_year)


past_algorithms <-
  bind_rows(df_preprocessed %>% filter(Year < 2020),
            lapply(past_years_to_include, function(y) get_latest_per_year(df_preprocessed, y)) %>% bind_rows())


total_algorithms_latest <- fromJSON(
  api_endpoint("/fred_dnssec_domains_by_algorithm_latest", zone = "eq.cz"
  )
) %>%
  mutate(
    Date = as.Date(ts),
    algorithm = as.factor(algorithm),
    Year = year(Date)
  )


total_algorithms <- rbind(total_algorithms_latest, past_algorithms)

df_algorithms <- total_algorithms %>%
  mutate(
    algorithm = fct_lump_n(algorithm, n = DISPLAYED_ALGO_COUNT, w = domains, other_level = gettext("Other")),
    Year = as.factor(Year)
  ) %>%
  group_by(Year, algorithm) %>%
  summarise(domains = sum(domains), .groups = "drop") %>%
  group_by(Year) %>%
  mutate(Total = sum(domains))

df_algorithms <- df_algorithms |>
  pivot_wider(
    names_from  = algorithm,
    values_from = domains
  ) |>
  mutate(across(everything(), ~ replace_na(.x, 0))) |>
  pivot_longer(
    cols = !c(Year, Total),
    names_to = "algorithm",
    values_to = "domains"
  ) |>
  mutate(
    domains_perc = (domains / Total) * 100
  )


plot_ly(
  df_algorithms,
  x = ~algorithm,
  y = ~domains_perc,
  frame = ~Year,
  type = "bar",
  showlegend = FALSE,
  marker =
    list(
      color = c(
        "#309DD3",
        "#186AB3",
        "#003893",
        "#61A3CE",
        "#F4587A",
        "#AA7DA4"
      )
    ),
  hoverinfo = "text",
  text = paste0(
    df_algorithms$Year, "<br>",
    "<b>", df_algorithms$algorithm, "</b>", "<br>",
    round(df_algorithms$domains_perc, 2), "% (",
    format(df_algorithms$domains, big.mark = " ", scientific = FALSE),
    " ", gettext("domains"), ")"
  ),
  textposition = "none"
) |>
  layout(
    xaxis =
      list(
        title = FALSE
      ),
    yaxis =
      list(
        title      = gettext("Shares of algorithms"),
        ticksuffix = "%"
      ),
    hovermode = "x unified"
  ) |>
  animation_opts(
    frame  = 1000,
    easing = "linear",
    redraw = FALSE
  ) |>
  animation_slider(
    currentvalue =
      list(
        prefix = paste0(
          gettext("Year"), ": "
        )
      )
  ) |>
  config(displayModeBar = TRUE, locale = locale_lang_code()) |>
  htmlwidgets::onRender(anim_skip_to_last_frame())
