df_dnssec <- fromJSON(api_endpoint("/cz_qps_edns0_do_1h"))

qps_dnssec_monthly <- df_dnssec |>
  mutate(
    date     = as.Date(ts),
    edns0_do = as.factor(edns0_do),
    edns0_do = if_else(edns0_do == 0, "NoDNSSEC", "DNSSEC"),
    year     = year(ts),
    month    = month(ts)
  ) |>
  group_by(month, year, edns0_do) |>
  summarize(mean_hourly_qps = mean(qps)) |>
  ungroup() |>
  arrange(year, month) |>
  filter(year != 2021 | month != 5) |> # remove incomplete data for 2021-05
  mutate(
    date = paste0(year, "-", month, "-01"),
    date = as.Date(date)
  ) |>
  pivot_wider(
    names_from = edns0_do,
    values_from = mean_hourly_qps
  ) |>
  mutate(
    total = DNSSEC + NoDNSSEC,
    DNSSEC_perc = (DNSSEC * 100) / total,
    noDNSSEC_perc = (NoDNSSEC * 100) / total,
    year = year(date),
    month = month(date)
  )

details(
  paste0(
    "<b>",
    gettext("QPS by DNSSEC"),
    "</b> – ",
    gettext("Proportional rate of monthly averages of hourly QPS averages by DNSSEC support")
  ), # nolint long string
  paste0(
    gettext("Whether a DNS query supports DNSSEC or not is determined from the EDNS0 flag") # nolint long string
  )
)

qps_dnssec_monthly |>
  plot_ly(
    type = "bar",
    x = ~date,
    y = ~DNSSEC_perc,
    marker = list(color = dashboard_colors[1]),
    name = gettext("DNSSEC"),
    hoverinfo = "text",
    text = paste0(
      "<b>", gettext("DNSSEC"), "</b>", ": ",
      round(qps_dnssec_monthly$DNSSEC_perc, 2), "% (hourly mean: ",
      round(qps_dnssec_monthly$DNSSEC, 2), " qps)"
    ),
    textposition = "none"
  ) |>
  add_trace(
    type = "bar",
    x = ~date,
    y = ~noDNSSEC_perc,
    marker = list(color = dashboard_colors[3]),
    name = gettext("No DNSSEC"),
    hoverinfo = "text",
    text = paste0(
      "<b>", gettext("No DNSSEC"), "</b>", ": ",
      round(qps_dnssec_monthly$noDNSSEC_perc, 2), "% (hourly mean: ",
      round(qps_dnssec_monthly$NoDNSSEC, 2), " qps)"
    ),
    textposition = "none"
  ) |>
  add_trace(
    type = "bar",
    x = ~date,
    y = 0,
    marker = list(color = "white"),
    name = gettext("DNSSEC"),
    hoverinfo = "text",
    text = paste0(
      qps_dnssec_monthly$year, "-", qps_dnssec_monthly$month
    ),
    textposition = "none",
    showlegend = FALSE
  ) |>
  layout(
    xaxis =
      list(
        title = gettext("Date")
      ),
    yaxis =
      list(
        title      = gettext("Percentage of queries"),
        ticksuffix = "%"
      ),
    legend =
      list(
        orientation = "h",
        xanchor     = "center",
        x           = 0.5,
        y           = 1.02
      ),
    barmode = "stack",
    hovermode = "x unified"
  ) |>
  config(displayModeBar = TRUE, locale = locale_lang_code()) |>
  htmlwidgets::onRender(add_csv_button_local())
