df_dnssec <- fromJSON(api_endpoint("/cz_qps_edns0_do_1h"))

qps_dnssec <- df_dnssec %>%
  mutate(
    ts = as.Date(ts),
    edns0_do = as.factor(edns0_do),
    edns0_do = ifelse(edns0_do == 0, "NO DNSSEC", "DNSSEC"),
    year = year(ts),
    month = month(ts),
    date = as.Date(paste0(year, "-", month, "-01"))
  ) %>%
  group_by(ts, edns0_do) %>%
  summarise(qps = mean(qps)) %>%
  mutate(total = sum(qps)) %>%
  ungroup() %>%
  spread(edns0_do, qps)

details(
  paste0(
    "<b>",
    gettext("Traffic rate by DNSSEC"),
    "</b> – ",
    gettext("time series of daily QPS averages by DNSSEC support")
  ),
  paste0(
    gettext("Whether a DNS query supports DNSSEC or not is determined from the EDNS0 flag"), # nolint long string
    " DNSSEC OK (DO) [<a href='https://datatracker.ietf.org/doc/html/rfc3225' target='_blank'>RFC 3225</a>]."
  )
) # nolint long string

plot_ly(qps_dnssec,
  x = ~ts,
  y = ~DNSSEC,
  type = "scatter",
  mode = "lines",
  line = list(color = dashboard_colors[1]),
  name = "DNSSEC",
  hovertemplate = paste0(
    "<b>DNSSEC</b>: %{y}<br><b>",
    gettext("frequency [%]"), "</b>: ",
    round(qps_dnssec$DNSSEC * 100 / qps_dnssec$total, 2),
    "<b><extra></extra>"
  )
) %>%
  add_trace(
    y = ~`NO DNSSEC`,
    type = "scatter",
    mode = "lines",
    line = list(color = dashboard_colors[3]),
    name = gettext("No DNSSEC"),
    hovertemplate = paste0(
      "<b>",
      gettext("No DNSSEC"), "</b>: %{y}<br><b>",
      gettext("frequency [%]"), "</b>: ",
      round(qps_dnssec$`NO DNSSEC` * 100 / qps_dnssec$total, 2),
      "<b><extra></extra>"
    )
  ) %>%
  config(locale = locale_lang_code()) %>%
  layout(
    hovermode = "x unified",
    xaxis = list(
      title = gettext("Date"),
      rangeselector = list(
        buttons = list(
          list(
            count = 1,
            label = gettext("1 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            count = 3,
            label = gettext("3 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            count = 6,
            label = gettext("6 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            step = "all",
            label = gettext("all")
          )
        ),
        rangeslider = list(type = "date")
      )
    ),
    yaxis = list(
      title = gettext("Queries per second")
    )
  )



df_dnssecv2 <- fromJSON(api_endpoint("/cz_qps_edns0_do_1h"))

qps_dnssecv2 <- df_dnssecv2 %>%
  mutate(
    ts = as.Date(ts),
    edns0_do = as.factor(edns0_do),
    edns0_do = ifelse(edns0_do == 0, "NO DNSSEC", "DNSSEC"),
    year = year(ts),
    month = month(ts)
  ) %>%
  group_by(month, year, edns0_do) %>%
  summarise(qps = mean(qps)) %>%
  mutate(
    total = sum(qps),
    date = paste0(year, "-", month, "-01")
  ) %>%
  ungroup() %>%
  mutate(date = as.Date(date))



plot_dnssec <- qps_dnssecv2 %>%
  ggplot(aes(date,
    qps,
    fill = edns0_do,
    text = sprintf(
      "%s<br>%s: <b>%s (%4.2f %%)</b>",
      date,
      edns0_do,
      formatC(qps, format = "d", big.mark = " "),
      qps / total * 100
    )
  )) +
  geom_bar(
    stat = "identity",
    position = position_fill(reverse = TRUE),
    color = dashboard_colors[1], size = 0.1
  ) +
  theme_classic() +
  xlab(gettext("Months")) +
  ylab(gettext("Shares of DNSSEC queries")) +
  scale_y_continuous(labels = scales::percent) +
  scale_fill_manual(values = dashboard_pal(2)) +
  theme(legend.title = element_blank()) +
  scale_x_date(date_breaks = "months", date_labels = "%b-%y")


hide <- ggplotly(plot_dnssec, tooltip = "text") %>%
  config(displayModeBar = FALSE) %>%
  layout(
    hovermode = "x unified",
    margin = list(t = 50),
    xaxis = list(fixedrange = TRUE)
  ) %>%
  layout(yaxis = list(fixedrange = TRUE))
