df_nia_means <-
  fromJSONwithToken(
    api_endpoint("/mojeid_day_users_nia_to_registration_means")
  )

df_nia_means <- df_nia_means |>
  mutate(
    ts = as.Date(ts),
    average_meantime = sub(" days? ", ":", average_meantime),
    median_meantime = sub(" days? ", ":", median_meantime)
  ) |>
  separate_wider_delim(average_meantime,
    delim = ":",
    too_few = "align_end",
    names = c(
      "mean_days",
      "mean_hours",
      "mean_minutes",
      "mean_seconds"
    )
  ) |>
  separate_wider_delim(median_meantime,
    delim = ":",
    too_few = "align_end",
    names = c(
      "median_days",
      "median_hours",
      "median_minutes",
      "median_seconds"
    )
  ) |>
  mutate(
    mean_days = as.integer(mean_days),
    mean_hours = as.integer(mean_hours),
    mean_minutes = as.integer(mean_minutes),
    mean_seconds = as.integer(mean_seconds),
    median_days = as.integer(median_days),
    median_hours = as.integer(median_hours),
    median_minutes = as.integer(median_minutes),
    median_seconds = as.integer(median_seconds)
  ) |>
  mutate(
    mean_days = replace_na(mean_days, 0),
    mean_hours = replace_na(mean_hours, 0),
    mean_minutes = replace_na(mean_minutes, 0),
    mean_seconds = replace_na(mean_seconds, 0),
    median_days = replace_na(median_days, 0),
    median_hours = replace_na(median_hours, 0),
    median_minutes = replace_na(median_minutes, 0),
    median_seconds = replace_na(median_seconds, 0)
  ) |>
  mutate(
    mean_minutes_s = mean_minutes * 60,
    mean_hours_s = mean_hours * 60 * 60,
    mean_days_s = mean_days * 60 * 60 * 24,
    mean_seconds_total =
      mean_seconds +
      mean_minutes_s +
      mean_hours_s +
      mean_days_s,
    median_minutes_s = median_minutes * 60,
    median_hours_s = median_hours * 60 * 60,
    median_days_s = median_days * 60 * 60 * 24,
    median_seconds_total =
      median_seconds +
      median_minutes_s +
      median_hours_s +
      median_days_s
  ) |>
  mutate(
    mean_time = as.duration(mean_seconds_total),
    median_time = as.duration(median_seconds_total)
  ) |>
  mutate(
    mean_time = as.period(mean_time),
    median_time = as.period(median_time)
  ) |>
  arrange(ts) |>
  mutate(weekday = weekdays(ts))

fig_nia_central <- plot_ly(
  df_nia_means,
  x = ~ts,
  y = ~median_days,
  type = "scatter",
  mode = "lines",
  name = gettext("Median"),
  line = list(color = dashboard_colors[1]),
  text = ~ paste0(
    "<b>", gettext("Median"), "</b>", ": ", median_days,
    " (", ts, ", ", weekday, ")"
  ),
  hoverinfo = "text",
  showlegend = FALSE
)
fig_nia_central <- fig_nia_central |>
  layout(
    hovermode = "x unified",
    xaxis =
      list(
        title = gettext("Date"),
        rangeselector = list(
          buttons = list(
            list(
              count = 7,
              label = gettext("7 d"),
              step = "day",
              stepmode = "backward"
            ),
            list(
              count = 1,
              label = gettext("1 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 2,
              label = gettext("2 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 6,
              label = gettext("6 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              step = "all",
              label = gettext("all")
            )
          ),
          rangeslider = list(type = "date")
        )
      ),
    yaxis =
      list(
        title = gettext("Days until connection to NIA (median)"),
        zeroline = FALSE
      ),
    xaxis =
      list(title = gettext("Date"))
  ) |>
  config(displayModeBar = TRUE, locale = locale_lang_code())


df_nia_aggregated <-
  fromJSONwithToken(
    api_endpoint("/mojeid_day_users_nia_to_registration_aggregated")
  )

df_nia_delay <- df_nia_aggregated |>
  rename(delay_time = category_name) |>
  mutate(
    ts = as.Date(ts),
    delay_time = as.factor(delay_time),
    group_percentage = sub("%", "", group_percentage),
    delay_time = gsub("Do mesice", "Do měsíce", delay_time),
    delay_time = gsub("Do tydne", "Do týdne", delay_time),
    delay_time = gsub("Pozdeji", "Později", delay_time),
    group_percentage = sub(",", ".", group_percentage)
  ) |>
  complete(
    ts, delay_time
  ) |>
  mutate_all(~ replace(., is.na(.), 0)) |>
  mutate(group_percentage = as.numeric(group_percentage)) |>
  mutate(
    weekday = weekdays(ts),
    weekday = recode(weekday,
      "Pondělí" = "Monday",
      "Úterý"   = "Tuesday",
      "Středa"  = "Wednesday",
      "Čtvrtek" = "Thursday",
      "Pátek"   = "Friday",
      "Sobota"  = "Saturday",
      "Neděle"  = "Sunday"
    ),
    delay_time = recode(delay_time,
      "Do hodiny" = "within_an_hour",
      "Do dne"    = "within_a_day",
      "Do týdne"  = "within_a_week",
      "Do měsíce" = "within_a_month",
      "Do roka"   = "within_a_year",
      "Do 5 let"  = "within_5_years",
      "Do 10 let" = "within_10_years",
      "Později"   = "later"
    )
  ) |>
  mutate(delay_time = factor(delay_time,
    levels = c(
      "within_an_hour",
      "within_a_day",
      "within_a_week",
      "within_a_month",
      "within_a_year",
      "within_5_years",
      "within_10_years",
      "later"
    )
  )) |>
  mutate(percentage_name = paste0(delay_time, "_percent")) |>
  select(
    ts,
    group_total,
    weekday,
    delay_time,
    group_percentage,
    percentage_name
  ) |>
  pivot_wider(
    names_from = delay_time,
    values_from = group_total,
    values_fn = list
  ) |>
  pivot_wider(
    names_from = percentage_name,
    values_from = group_percentage
  ) |>
  group_by(ts) |>
  summarize(
    weekday = first(weekday),
    within_an_hour = first(within_an_hour, na_rm = TRUE),
    within_a_day = first(within_a_day, na_rm = TRUE),
    within_a_week = first(within_a_week, na_rm = TRUE),
    within_a_month = first(within_a_month, na_rm = TRUE),
    within_a_year = first(within_a_year, na_rm = TRUE),
    within_5_years = first(within_5_years, na_rm = TRUE),
    within_10_years = first(within_10_years, na_rm = TRUE),
    later = first(later, na_rm = TRUE),
    within_an_hour_percent = first(round(within_an_hour_percent, 2), na_rm = TRUE),
    within_a_day_percent = first(round(within_a_day_percent, 2), na_rm = TRUE),
    within_a_week_percent = first(round(within_a_week_percent, 2), na_rm = TRUE),
    within_a_month_percent = first(round(within_a_month_percent, 2), na_rm = TRUE),
    within_a_year_percent = first(round(within_a_year_percent, 2), na_rm = TRUE),
    within_5_years_percent = first(round(within_5_years_percent, 2), na_rm = TRUE),
    within_10_years_percent = first(round(within_10_years_percent, 2), na_rm = TRUE),
    later_percent = first(round(later_percent, 2), na_rm = TRUE)
  ) |>
  ungroup() |>
  mutate_all(~ replace(., is.na(.), 0)) |>
  arrange(ymd(ts))

fig_nia_delay <- plot_ly(
  df_nia_delay,
  x = ~ts,
  y = ~later,
  type = "bar",
  name = gettext("Later"),
  marker = list(color = "#003893"),
  text = ~ paste0(
    "<b>", gettext("Later"),
    "</b>: ", later,
    " (", later_percent, "%)"
  ),
  hoverinfo = "text",
  textposition = "none"
)
fig_nia_delay <- fig_nia_delay |>
  add_trace(
    x = ~ts,
    y = ~within_10_years,
    type = "bar",
    name = gettext("Within 10 years"),
    marker = list(color = "#115CAA"),
    text = ~ paste0(
      "<b>", gettext("Within 10 years"),
      "</b>: ", within_10_years,
      " (", within_10_years_percent, "%)"
    ),
    hoverinfo = "text",
    textposition = "none"
  )
fig_nia_delay <- fig_nia_delay |>
  add_trace(
    x = ~ts,
    y = ~within_5_years,
    type = "bar",
    name = gettext("Within 5 years"),
    marker = list(color = "#2280C1"),
    text = ~ paste0(
      "<b>", gettext("Within 5 years"),
      "</b>: ", within_5_years,
      " (", within_5_years_percent, "%)"
    ),
    hoverinfo = "text",
    textposition = "none"
  )
fig_nia_delay <- fig_nia_delay |>
  add_trace(
    x = ~ts,
    y = ~within_a_year,
    type = "bar",
    name = gettext("Within a year"),
    marker = list(color = "#34A4D8"),
    text = ~ paste0(
      "<b>", gettext("Within a year"),
      "</b>: ", within_a_year,
      " (", within_a_year_percent, "%)"
    ),
    hoverinfo = "text",
    textposition = "none"
  )
fig_nia_delay <- fig_nia_delay |>
  add_trace(
    x = ~ts,
    y = ~within_a_month,
    type = "bar",
    name = gettext("Within a month"),
    marker = list(color = "#57A9D4"),
    text = ~ paste0(
      "<b>", gettext("Within a month"),
      "</b>: ", within_a_month,
      " (", within_a_month_percent, "%)"
    ),
    hoverinfo = "text",
    textposition = "none"
  )
fig_nia_delay <- fig_nia_delay |>
  add_trace(
    x = ~ts,
    y = ~within_a_week,
    type = "bar",
    name = gettext("Within a week"),
    marker = list(color = "#8B8EB6"),
    text = ~ paste0(
      "<b>", gettext("Within a week"),
      "</b>: ", within_a_week,
      " (", within_a_week_percent, "%)"
    ),
    hoverinfo = "text",
    textposition = "none"
  )
fig_nia_delay <- fig_nia_delay |>
  add_trace(
    x = ~ts,
    y = ~within_a_day,
    type = "bar",
    name = gettext("Within a day"),
    marker = list(color = "#BF7398"),
    text = ~ paste0(
      "<b>", gettext("Within a day"),
      "</b>: ", within_a_day,
      " (", within_a_day_percent, "%)"
    ),
    hoverinfo = "text",
    textposition = "none"
  )
fig_nia_delay <- fig_nia_delay |>
  add_trace(
    x = ~ts,
    y = ~within_an_hour,
    type = "bar",
    name = gettext("Within an hour"),
    marker = list(color = "#F4587A"),
    text = ~ paste0(
      "<b>", gettext("Within an hour"),
      "</b>: ", within_an_hour,
      " (", within_an_hour_percent, "%)"
    ),
    hoverinfo = "text",
    textposition = "none"
  )
fig_nia_delay <- fig_nia_delay |>
  add_trace(
    x = ~ts,
    y = 0,
    name = gettext("Date"),
    marker = list(color = "white"),
    text = ~ paste0(
      "<b>", gettext("Date"), "</b>: ", ts
    ),
    showlegend = FALSE
  )
fig_nia_delay <- fig_nia_delay |>
  layout(
    hovermode = "x unified",
    barmode = "stack",
    legend = list(
      orientation = "v",
      xanchor     = "left",
      yanchor     = "center",
      x           = 1,
      y           = 1
    ),
    yaxis =
      list(title = gettext("Count of validations")),
    xaxis =
      list(
        title = gettext("Date"),
        rangeselector = list(
          buttons = list(
            list(
              count = 7,
              label = gettext("7 d"),
              step = "day",
              stepmode = "backward"
            ),
            list(
              count = 1,
              label = gettext("1 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 2,
              label = gettext("2 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 6,
              label = gettext("6 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              step = "all",
              label = gettext("all")
            )
          ),
          rangeslider = list(type = "date")
        )
      )
  ) |>
  config(displayModeBar = TRUE, locale = locale_lang_code())

fig_nia_delayplots <-
  subplot(fig_nia_central, fig_nia_delay,
    margin = 0.06,
    titleX = TRUE,
    titleY = TRUE,
    nrows  = 1
  ) |>
  layout(
    autosize = TRUE,
    xaxis = list(showgrid = TRUE, zeroline = FALSE),
    yaxis = list(showgrid = TRUE, zeroline = FALSE)
  ) |>
  htmlwidgets::onRender(add_csv_button_local())
fig_nia_delayplots
