endpoint <- api_endpoint("/mojeid_accounts")
api_data <- fromJSON(endpoint)

df_mojeid_accounts <- api_data %>%
  mutate(ts = as.Date(ts), year = year(ts)) %>%
  group_by(ts, year, highest_level) %>%
  summarise(count = sum(count)) %>%
  mutate(
    Total = sum(count),
    percent = round((count / Total * 100), 2)
  ) %>%
  ungroup()

df_mojeid_accounts_2020 <- df_mojeid_accounts %>%
  filter(year == "2020") %>%
  filter(ts == max(ts))

df_mojeid_accounts_2021 <- df_mojeid_accounts %>%
  filter(year >= "2021") %>%
  mutate(
    ts = as.Date(ts),
    year = format(ts, "%Y"),
    Day = days_in_month(ts),
    day_logins = day(ts)
  ) %>%
  filter(Day == day_logins) %>%
  group_by(year, highest_level) %>%
  filter(ts == max(ts)) %>%
  select(-Day, -day_logins)

df_mojeid_accounts_total <- df_mojeid_accounts %>%
  filter(year < 2020) %>%
  rbind(df_mojeid_accounts_2020, df_mojeid_accounts_2021) %>%
  ungroup()

a <- df_mojeid_accounts_total %>%
  filter(highest_level == "conditionallyIdentifiedContact") %>%
  group_by(ts, year, Total, highest_level) %>%
  summarise(cumsum = 0) %>%
  mutate(label = "") %>%
  ungroup()

b <- df_mojeid_accounts_total %>%
  filter(highest_level %in% c(
    "conditionallyIdentifiedContact",
    "identifiedContact"
  )) %>%
  group_by(ts, year, Total) %>%
  summarise(cumsum = sum(count)) %>%
  mutate(
    highest_level = "identifiedContact",
    label = "CIC + IC"
  ) %>%
  ungroup()

c <- df_mojeid_accounts_total %>%
  group_by(ts, year, Total) %>%
  summarise(cumsum = sum(count)) %>%
  mutate(
    highest_level = "validatedContact",
    label = "CIC + IC + VC"
  ) %>%
  ungroup()

cumulative_sum <- rbind(a, b, c)

df_mojeid_accounts_cumsum <-
  inner_join(df_mojeid_accounts_total,
    cumulative_sum,
    by = c("ts", "year", "highest_level", "Total")
  ) %>%
  mutate(abbr = ifelse(
    highest_level == "conditionallyIdentifiedContact",
    "CIC",
    ifelse(
      highest_level == "identifiedContact",
      "IC", ("VC")
    )
  ))

data <- df_mojeid_accounts_cumsum %>%
  spread(highest_level, count)


fig <- plot_ly(data,
  x = ~year,
  y = ~conditionallyIdentifiedContact,
  type = "bar",
  marker = list(color = dashboard_colors[1]),
  name = gettext("Conditionally Identified Contact (CIC)"),
  hovertemplate = paste0(
    "<b>", "CIC",  "</b>: %{y}", " (", data$percent, "%)",
    "<extra></extra>"
  )
)
fig <- fig %>% add_trace(
  y = ~identifiedContact,
  name = gettext("Identified Contact (IC)"),
  marker = list(color = dashboard_colors[2]),
  hovertemplate = paste0(
    "<b>", "IC", "</b>: %{y}", " (", data$percent, "%)<br>",
    "<b>", "CIC + IC", "</b>: ",
    formatC(data$cumsum, format = "d", big.mark = "\u202F"),
    "<extra></extra>"
  )
)
fig <- fig %>% add_trace(
  y = ~validatedContact,
  name = gettext("Validated Contact (VC)"),
  marker = list(color = dashboard_colors[3]),
  hovertemplate = paste0(
    "<b>", "VC", "</b>: %{y}", " (", data$percent, "%)<br>",
    "<b>", "CIC + IC", "</b>: ",
    formatC(data$cumsum, format = "d", big.mark = "\u202F"),
    "<extra></extra>"
  )
)
fig <- fig %>%
  config(
    displayModeBar = TRUE,
    locale = locale_lang_code()
  ) %>%
  htmlwidgets::onRender(add_csv_button(endpoint)) %>%
  layout(
    barmode = "stack",
    hovermode = "x unified",
    title = gettext("Number of MojeID accounts"),
    legend = list(
      orientation = "h",
      xanchor = "center",
      yanchor = "bottom",
      x = 0.5,
      y = 0.99
    ),
    separators = ".\u202F",
    margin = list(t = 50),
    xaxis = list(
      fixedrange = TRUE,
      dtick = 1,
      title = gettext("Year")
    )
  ) %>%
  layout(yaxis = list(
    fixedrange = TRUE,
    title = gettext("Accounts"),
    dtick = 40000,
    tickformat = ",d"
  ))


fig
