library(flexdashboard)
library(dplyr)
library(gridExtra)
library(ggplot2)
library(plotly)
library(lubridate)
library(tidyr)
library(knitr)
library(kableExtra)
library(formattable)
library(sparkline)
library(jsonlite)
library(forcats)

#---- plot functions ----

#' Plot a sankey diagram for Country -> DNS server flows
#'
#' @param df_qps
#' @param df_locations
#' @param n
plot_sankey_traffic <- function(df,
                                n = 15,
                                pal_clients = c("white", dashboard_colors[1]),
                                pal_servers = c("white", dashboard_colors[3])) {
  df <- df %>%
    group_by(client_country, server_location) %>%
    summarise(qps = mean(qps)) %>%
    ungroup()

  df <- df %>%
    replace_na(list(
      client_country = gettext("unknown"),
      server_location = gettext("undisclosed")
    ))

  qps_cc <- df %>%
    group_by(client_country) %>%
    summarise(qps_sum = sum(qps)) %>%
    arrange(desc(qps_sum))

  df <- df %>%
    mutate(client_country = fct_other(client_country,
      other_level = gettext("other"),
      keep = qps_cc$client_country[1:n]
    )) %>%
    group_by(client_country, server_location) %>%
    summarise(qps = sum(qps)) %>%
    ungroup()

  df_clients <- df %>%
    group_by(client_country) %>%
    summarise(qps = sum(qps)) %>%
    mutate(color = csscolor(gradient(
      as.numeric(qps),
      pal_clients[1], pal_clients[2]
    ))) %>%
    rename(label = "client_country")
  df_servers <- df %>%
    group_by(server_location) %>%
    summarise(qps = sum(qps)) %>%
    mutate(color = csscolor(gradient(
      as.numeric(qps),
      pal_servers[1], pal_servers[2]
    ))) %>%
    rename(label = "server_location")
  nodes <- c(as.character(df_clients$label), df_servers$label)

  n_clients <- nrow(df_clients)

  colors <- c(df_clients$color, df_servers$color)

  df_links <- data.frame(
    from = match(df$client_country, df_clients$label) - 1,
    to = match(df$server_location, df_servers$label) + n_clients - 1,
    qps = df$qps
  )

  fig <- plot_ly(
    type = "sankey",
    domain = list(
      x =  c(0, 1),
      y =  c(0, 1)
    ),
    orientation = "h",
    valueformat = ifelse(min(df_clients$qps) > 1, ".0f", ".1f"),
    valuesuffix = " QPS",
    node = list(
      label = nodes,
      pad = 15,
      color = colors,
      thickness = 15,
      hovertemplate = "%{label}",
      line = list(
        color = "black",
        width = 0.5
      )
    ),
    link = list(
      source = df_links$from,
      target = df_links$to,
      value =  df_links$qps
    )
  ) %>%
    config(locale = locale_lang_code()) %>%
    htmlwidgets::onRender(add_csv_button(endpoint))

  fig <- fig %>% layout(
    margin = list(
      l = 4,
      r = 4
    ),
    font = list(
      size = 12
    ),
    xaxis = list(showgrid = FALSE, zeroline = FALSE),
    yaxis = list(showgrid = FALSE, zeroline = FALSE)
  )
  fig
}

#' IP versions Timeserie
#' @param endpoint
#' @param df qps_ipv
qps_ipv_plot <- function(df, endpoint) {
  qps_ipv <- df %>%
    mutate(
      ts = as.Date(ts), ts = as.Date(ts),
      ipv = as.factor(ipv)
    ) %>%
    group_by(ts, ipv) %>%
    summarise(qps = mean(qps)) %>%
    spread(ipv, qps) %>%
    rename(IPv4 = `4`, IPv6 = `6`) %>%
    ungroup()

  plot_ly(
    qps_ipv,
    x = ~ts,
    y = ~IPv4,
    mode = "lines",
    line = list(color = dashboard_colors[1]),
    name = "IPv4",
    hovertemplate = paste0(
      "<b>IPv4</b>: %{y} qps<extra></extra>"
    )
  ) %>%
    add_trace(
      x = ~ts,
      y = ~IPv6,
      line = list(color = dashboard_colors[3]),
      name = "IPv6",
      hovertemplate = paste0(
        "<b>IPv6</b>: %{y} qps<extra></extra>"
      )
    ) %>%
    add_lines() %>%
    config(locale = locale_lang_code()) %>%
    onRender(add_csv_button(
      api_endpoint(endpoint),
      series = "ipv",
      mapping = csv_mapping_ipv
    )) %>%
    layout(
      hovermode = "x unified",
      xaxis = list(
        title = gettext("Date"),
        rangeselector = list(
          buttons = list(
            list(
              count = 1,
              label = gettext("1 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 3,
              label = gettext("3 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 6,
              label = gettext("6 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              step = "all",
              label = gettext("all")
            )
          ),
          rangeslider = list(type = "date")
        )
      ),
      yaxis = list(title = gettext("Queries per second"))
    )
}

# Protocol timeserie
#' @param endpoint
#' @param df qps_protocol
qps_protocol_plot <- function(df, endpoint) {
  qps_protocol <- df %>%
    mutate(
      ts = as.Date(ts),
      protocol = as.factor(prot),
      Protocol = ifelse(prot == 6, "TCP", "UDP")
    ) %>%
    group_by(ts, Protocol) %>%
    summarise(qps = mean(qps)) %>%
    spread(Protocol, qps) %>%
    ungroup()

  plot_ly(qps_protocol,
    x = ~ts,
    y = ~TCP,
    mode = "lines",
    line = list(color = dashboard_colors[3]),
    name = "TCP",
    hovertemplate = paste0(
      "<b>TCP</b>: %{y} qps<extra></extra>"
    )
  ) %>%
    add_trace(
      x = ~ts,
      y = ~UDP,
      line = list(color = dashboard_colors[1]),
      name = "UDP",
      hovertemplate = paste0(
        "<b>UDP</b>: %{y} qps<extra></extra>"
      )
    ) %>%
    add_lines() %>%
    config(locale = locale_lang_code()) %>%
    onRender(add_csv_button(
      api_endpoint(endpoint),
      series = "prot",
      mapping = csv_mapping_prot
    )) %>%
    layout(
      hovermode = "x unified",
      xaxis = list(
        title = gettext("Date"),
        rangeselector = list(
          buttons = list(
            list(
              count = 1,
              label = gettext("1 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 3,
              label = gettext("3 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 6,
              label = gettext("6 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              step = "all",
              label = gettext("all")
            )
          ),
          rangeslider = list(type = "date")
        )
      ),
      yaxis = list(title = gettext("Queries per second"))
    )
}

# Server tab table
#' @param server_locations
#' @param df qps_server_1h

qps_server_plot <- function(df, server_locations) {
  qps_server_1h <- df %>%
    left_join(server_cc, by = c("server" = "server_name")) %>%
    mutate(city = ifelse(is.na(city), gettext("undisclosed"), city))

  df_qps_server_traffic <- qps_server_1h %>%
    mutate(city = as.factor(city), ts = as.Date(ts)) %>%
    group_by(ts, city) %>%
    summarise(qps = sum(qps)) %>%
    mutate(queries = qps * 3600, queries_24h = queries / 86400)


  df_qps_server_traffic_table <- df_qps_server_traffic %>%
    group_by(city) %>%
    summarise(
      qps_max = round(max(queries_24h), 0),
      qps_min = round(min(queries_24h), 0)
    ) %>%
    ungroup() %>%
    mutate(
      qps_max_tile = adam_color_tile(qps_max, "white", dashboard_colors[1]),
      qps_min_tile = adam_color_tile(qps_min, "white", dashboard_colors[1])
    ) %>%
    inner_join(
      df_qps_server_traffic %>%
        group_by(city) %>%
        summarise(
          qps_current = mean(queries_24h)
        ) %>%
        ungroup() %>%
        mutate(
          qps_current_tile = adam_color_tile(
            qps_current,
            "white", dashboard_colors[3]
          )
        ),
      by = c("city")
    )

  df_qps_server_traffic_spark <- df_qps_server_traffic %>%
    arrange(ts) %>%
    group_by(city) %>%
    summarise(
      queries_bar = spk_chr(
        queries_24h,
        type = "line",
        lineColor = sparkline_colors[1],
        fillColor = sparkline_colors[2],
        spotColor = TRUE,
        width = 80
      )
    )


  htmltools::attachDependencies(
    htmltools::div(),
    html_dependency_sparkline()
  )


  df_qps_server_traffic_table %>%
    left_join(df_qps_server_traffic_spark, by = "city") %>%
    arrange(desc(qps_current)) %>%
    select(
      city,
      qps_max_tile,
      qps_min_tile,
      queries_bar,
      qps_current_tile
    ) %>%
    kable(
      "html",
      table.attr = "class='dtable'",
      escape = FALSE,
      align = c("r", "c", "c", "c", "c"),
      format.args = list(big.mark = " "),
      col.names = c(
        gettext("City"), "Max QPS", "Min QPS",
        "Trend", gettext("Average")
      )
    ) %>%
    column_spec(1, bold = TRUE, width = "3cm") %>%
    column_spec(2, width = "3cm") %>%
    column_spec(3, width = "3cm") %>%
    column_spec(4, width = "3cm") %>%
    column_spec(5, width = "3cm") %>%
    kable_styling(
      bootstrap_options = c("striped", "hover", "condensed", "responsive"),
      full_width = TRUE,
      position = "left"
    )
}

# Rcode tab table
#' @param rcode
#' @param df qps_rcode_1h

qps_rcode_plot <- function(df, rcode) {
  qps_rcode <- df %>% inner_join(rcode, by = c("rcode"))

  df_qps_rcode_traffic <- qps_rcode %>%
    mutate(ts = as.Date(ts)) %>%
    group_by(ts, Name) %>%
    summarise(queries = sum(qps)) %>%
    mutate(queries = queries * 3600) %>%
    mutate(queries_24h = queries / 86400) %>%
    group_by(ts, Name) %>%
    summarise(queries_24h = sum(queries_24h))

  df_qps_rcode_traffic_table <- df_qps_rcode_traffic %>%
    group_by(Name) %>%
    summarise(
      qps_max = round(max(queries_24h), 0),
      qps_min = round(min(queries_24h), 0)
    ) %>%
    ungroup() %>%
    mutate(
      qps_max_tile = adam_color_tile(qps_max, "white", dashboard_colors[1]),
      qps_min_tile = adam_color_tile(qps_min, "white", dashboard_colors[1])
    ) %>%
    inner_join(
      df_qps_rcode_traffic %>%
        group_by(Name) %>%
        summarise(
          qps_current = mean(queries_24h)
        ) %>%
        ungroup() %>%
        mutate(
          qps_current_tile = adam_color_tile(
            qps_current,
            "white", dashboard_colors[3]
          )
        ),
      by = c("Name")
    )

  df_qps_rcode_traffic_sparkline <- df_qps_rcode_traffic %>%
    group_by(Name) %>%
    arrange(ts) %>%
    summarise(
      queries_bar = spk_chr(
        queries_24h,
        type = "line",
        lineColor = sparkline_colors[1],
        fillColor = sparkline_colors[2],
        spotColor = TRUE,
        width = 80
      )
    )

  htmltools::attachDependencies(
    htmltools::div(),
    html_dependency_sparkline()
  )

  df_qps_rcode_traffic_table %>%
    left_join(df_qps_rcode_traffic_sparkline, by = "Name") %>%
    arrange(desc(qps_current)) %>%
    select(
      Name,
      qps_max_tile,
      qps_min_tile,
      queries_bar,
      qps_current_tile
    ) %>%
    kable(
      "html",
      table.attr = "class='dtable'",
      escape = FALSE,
      align = c("r", "c", "c", "c", "c"),
      format.args = list(big.mark = " "),
      col.names = c(
        "RCODE", "Max QPS", "Min QPS",
        "Trend", gettext("Average")
      )
    ) %>%
    column_spec(1, bold = TRUE, width = "3cm") %>%
    column_spec(2, width = "3cm") %>%
    column_spec(3, width = "3cm") %>%
    column_spec(4, width = "3cm") %>%
    column_spec(5, width = "3cm") %>%
    kable_styling(
      bootstrap_options = c("striped", "hover", "condensed", "responsive"),
      full_width = TRUE,
      position = "left"
    )
}

# Rcode tab table for double qps values
#' @param rcode
#' @param df qps_rcode_1h
#' @param int decimals

qps_rcode_plot2 <- function(df, rcode, decimals) {
  qps_rcode <- df %>% inner_join(rcode, by = c("rcode"))

  df_qps_rcode_traffic <- qps_rcode %>%
    mutate(ts = as.Date(ts)) %>%
    group_by(ts, Name) %>%
    summarise(queries = sum(qps)) %>%
    mutate(queries = queries * 3600) %>%
    mutate(queries_24h = queries / 86400) %>%
    group_by(ts, Name) %>%
    summarise(queries_24h = sum(queries_24h))

  df_qps_rcode_traffic_table <- df_qps_rcode_traffic %>%
    group_by(Name) %>%
    summarise(
      qps_max = round(max(queries_24h), decimals),
      qps_min = round(min(queries_24h), decimals)
    ) %>%
    ungroup() %>%
    mutate(
      qps_max_tile = adam_color_tile2(
        qps_max, "white",
        dashboard_colors[1],
        decimals
      ),
      qps_min_tile = adam_color_tile2(
        qps_min, "white",
        dashboard_colors[1],
        decimals
      )
    ) %>%
    inner_join(
      df_qps_rcode_traffic %>%
        group_by(Name) %>%
        summarise(
          qps_current = mean(queries_24h)
        ) %>%
        ungroup() %>%
        mutate(
          qps_current_tile = adam_color_tile2(
            qps_current,
            "white", dashboard_colors[3],
            decimals
          )
        ),
      by = c("Name")
    )

  df_qps_rcode_traffic_sparkline <- df_qps_rcode_traffic %>%
    group_by(Name) %>%
    arrange(ts) %>%
    summarise(
      queries_bar = spk_chr(
        queries_24h,
        type = "line",
        lineColor = sparkline_colors[1],
        fillColor = sparkline_colors[2],
        spotColor = TRUE,
        width = 80
      )
    )

  htmltools::attachDependencies(
    htmltools::div(),
    html_dependency_sparkline()
  )

  df_qps_rcode_traffic_table %>%
    left_join(df_qps_rcode_traffic_sparkline, by = "Name") %>%
    arrange(desc(qps_current)) %>%
    select(
      Name,
      qps_max_tile,
      qps_min_tile,
      queries_bar,
      qps_current_tile
    ) %>%
    kable(
      "html",
      table.attr = "class='dtable'",
      escape = FALSE,
      align = c("r", "c", "c", "c", "c"),
      format.args = list(big.mark = " "),
      col.names = c(
        "RCODE", "Max QPS", "Min QPS",
        "Trend", gettext("Average")
      )
    ) %>%
    column_spec(1, bold = TRUE, width = "3cm") %>%
    column_spec(2, width = "3cm") %>%
    column_spec(3, width = "3cm") %>%
    column_spec(4, width = "3cm") %>%
    column_spec(5, width = "3cm") %>%
    kable_styling(
      bootstrap_options = c("striped", "hover", "condensed", "responsive"),
      full_width = TRUE,
      position = "left"
    )
}

# Qtype tab table
#' @param qtypes
#' @param df qps_qtype_1h
#' @param num fct_lump
qps_qtype_plot <- function(df, qtypes, num = 15) {
  qps_qtype <- df %>% inner_join(qtypes, by = c("Value"))

  top_qtypes_latest <- qps_qtype %>%
    group_by(ts, TYPE) %>%
    summarise(qps = sum(qps)) %>%
    mutate(TYPE = as.factor(TYPE)) %>%
    mutate(TYPE = fct_lump_n(TYPE,
      n = num, w = qps,
      other_level = gettext("other")
    )) %>%
    group_by(TYPE) %>%
    summarise(qps = sum(qps))

  df_top_qtypes <- qps_qtype %>%
    mutate(ts = as.Date(ts)) %>%
    mutate(TYPE = as.factor(TYPE), queries = qps * 3600) %>%
    group_by(ts, TYPE) %>%
    summarise(queries = sum(queries)) %>%
    inner_join(top_qtypes_latest, by = c("TYPE")) %>%
    select(ts, TYPE, queries)

  df_qps_qtype_traffic <- qps_qtype %>%
    mutate(ts = as.Date(ts)) %>%
    mutate(TYPE = as.factor(TYPE), queries = qps * 3600) %>%
    group_by(ts, TYPE) %>%
    summarise(queries = sum(queries)) %>%
    anti_join(top_qtypes_latest, by = c("TYPE")) %>%
    mutate(TYPE = gettext("other")) %>%
    group_by(ts, TYPE) %>%
    summarise(queries = sum(queries)) %>%
    bind_rows(df_top_qtypes) %>%
    mutate(queries_24h = queries / 86400)

  df_qps_qtype_traffic_table <- df_qps_qtype_traffic %>%
    group_by(TYPE) %>%
    summarise(
      qps_max = round(max(queries_24h), 0),
      qps_min = round(min(queries_24h), 0)
    ) %>%
    ungroup() %>%
    mutate(
      qps_max_tile = adam_color_tile(qps_max, "white", dashboard_colors[1]),
      qps_min_tile = adam_color_tile(qps_min, "white", dashboard_colors[1])
    ) %>%
    inner_join(
      df_qps_qtype_traffic %>%
        group_by(TYPE) %>%
        summarise(
          qps_current = mean(queries_24h)
        ) %>%
        ungroup() %>%
        mutate(
          qps_current_tile = adam_color_tile(
            qps_current,
            "white", dashboard_colors[3]
          )
        ),
      by = c("TYPE")
    )

  df_qps_qtype_traffic_sparkline <- df_qps_qtype_traffic %>%
    group_by(ts, TYPE) %>%
    summarise(qps = mean(queries_24h)) %>%
    arrange(ts) %>%
    group_by(TYPE) %>%
    summarise(
      queries_bar = spk_chr(
        qps,
        type = "line",
        lineColor = sparkline_colors[1],
        fillColor = sparkline_colors[2],
        spotColor = TRUE,
        width = 80
      )
    )

  htmltools::attachDependencies(
    htmltools::div(),
    html_dependency_sparkline()
  )

  df_qps_qtype_traffic_table %>%
    left_join(df_qps_qtype_traffic_sparkline, by = "TYPE") %>%
    arrange(desc(qps_current)) %>%
    select(
      TYPE,
      qps_max_tile,
      qps_min_tile,
      queries_bar,
      qps_current_tile
    ) %>%
    kable(
      "html",
      table.attr = "class='dtable'",
      escape = FALSE,
      align = c("r", "c", "c", "c", "c"),
      format.args = list(big.mark = " "),
      col.names = c(
        "QTYPE", "Max qps", "Min qps",
        "Trend", gettext("Average")
      )
    ) %>%
    column_spec(1, bold = TRUE, width = "3cm") %>%
    column_spec(2, width = "3cm") %>%
    column_spec(3, width = "3cm") %>%
    column_spec(4, width = "3cm") %>%
    column_spec(5, width = "3cm") %>%
    kable_styling(
      bootstrap_options = c("striped", "hover", "condensed", "responsive"),
      full_width = TRUE,
      position = "left"
    )
}

# Qtype tab table with decimals
#' @param qtypes
#' @param df qps_qtype_1h
#' @param num fct_lump
#' @param int decimals
qps_qtype_plot2 <- function(df, qtypes, num = 15, decimals) {
  qps_qtype <- df %>% inner_join(qtypes, by = c("Value"))

  top_qtypes_latest <- qps_qtype %>%
    group_by(ts, TYPE) %>%
    summarise(qps = sum(qps)) %>%
    mutate(TYPE = as.factor(TYPE)) %>%
    mutate(TYPE = fct_lump_n(TYPE,
      n = num, w = qps,
      other_level = gettext("other")
    )) %>%
    group_by(TYPE) %>%
    summarise(qps = sum(qps))

  df_top_qtypes <- qps_qtype %>%
    mutate(ts = as.Date(ts)) %>%
    mutate(TYPE = as.factor(TYPE), queries = qps * 3600) %>%
    group_by(ts, TYPE) %>%
    summarise(queries = sum(queries)) %>%
    inner_join(top_qtypes_latest, by = c("TYPE")) %>%
    select(ts, TYPE, queries)

  df_qps_qtype_traffic <- qps_qtype %>%
    mutate(ts = as.Date(ts)) %>%
    mutate(TYPE = as.factor(TYPE), queries = qps * 3600) %>%
    group_by(ts, TYPE) %>%
    summarise(queries = sum(queries)) %>%
    anti_join(top_qtypes_latest, by = c("TYPE")) %>%
    mutate(TYPE = gettext("other")) %>%
    group_by(ts, TYPE) %>%
    summarise(queries = sum(queries)) %>%
    bind_rows(df_top_qtypes) %>%
    mutate(queries_24h = queries / 86400)

  df_qps_qtype_traffic_table <- df_qps_qtype_traffic %>%
    group_by(TYPE) %>%
    summarise(
      qps_max = round(max(queries_24h), decimals),
      qps_min = round(min(queries_24h), decimals)
    ) %>%
    ungroup() %>%
    mutate(
      qps_max_tile = adam_color_tile2(
        qps_max,
        "white",
        dashboard_colors[1],
        decimals
      ),
      qps_min_tile = adam_color_tile2(
        qps_min,
        "white",
        dashboard_colors[1],
        decimals
      )
    ) %>%
    inner_join(
      df_qps_qtype_traffic %>%
        group_by(TYPE) %>%
        summarise(
          qps_current = mean(queries_24h)
        ) %>%
        ungroup() %>%
        mutate(
          qps_current_tile = adam_color_tile2(
            qps_current,
            "white",
            dashboard_colors[3],
            decimals
          )
        ),
      by = c("TYPE")
    )

  df_qps_qtype_traffic_sparkline <- df_qps_qtype_traffic %>%
    group_by(ts, TYPE) %>%
    summarise(qps = mean(queries_24h)) %>%
    arrange(ts) %>%
    group_by(TYPE) %>%
    summarise(
      queries_bar = spk_chr(
        qps,
        type = "line",
        lineColor = sparkline_colors[1],
        fillColor = sparkline_colors[2],
        spotColor = TRUE,
        width = 80
      )
    )

  htmltools::attachDependencies(
    htmltools::div(),
    html_dependency_sparkline()
  )

  df_qps_qtype_traffic_table %>%
    left_join(df_qps_qtype_traffic_sparkline, by = "TYPE") %>%
    arrange(desc(qps_current)) %>%
    select(
      TYPE,
      qps_max_tile,
      qps_min_tile,
      queries_bar,
      qps_current_tile
    ) %>%
    kable(
      "html",
      table.attr = "class='dtable'",
      escape = FALSE,
      align = c("r", "c", "c", "c", "c"),
      format.args = list(big.mark = " "),
      col.names = c(
        "QTYPE", "Max qps", "Min qps",
        "Trend", gettext("Average")
      )
    ) %>%
    column_spec(1, bold = TRUE, width = "3cm") %>%
    column_spec(2, width = "3cm") %>%
    column_spec(3, width = "3cm") %>%
    column_spec(4, width = "3cm") %>%
    column_spec(5, width = "3cm") %>%
    kable_styling(
      bootstrap_options = c("striped", "hover", "condensed", "responsive"),
      full_width = TRUE,
      position = "left"
    )
}

# Resolvers tab table
#' @param df qps_cc_1h
#' @param num fct_lump
qps_resolvers_plot <- function(df, num = 15) {
  qps_cc <- df %>% inner_join(country_codes, by = c("country_code"))
  df_qps_cc_traffic <- qps_cc %>%
    mutate(
      Country = as.factor(Country), ts = as.Date(ts),
      Country = fct_lump_n(Country,
        n = num, w = qps,
        other_level = gettext("other")
      )
    ) %>%
    group_by(ts, Country) %>%
    summarise(qps = sum(qps)) %>%
    mutate(queries = qps * 3600, queries_24h = queries / 86400)

  df_qps_cc_traffic_table <- df_qps_cc_traffic %>%
    group_by(Country) %>%
    summarise(
      qps_max = round(max(queries_24h), 0),
      qps_min = round(min(queries_24h), 0)
    ) %>%
    ungroup() %>%
    mutate(
      qps_max_tile = adam_color_tile(qps_max, "white", dashboard_colors[1]),
      qps_min_tile = adam_color_tile(qps_min, "white", dashboard_colors[1])
    ) %>%
    inner_join(
      df_qps_cc_traffic %>%
        group_by(Country) %>%
        summarise(
          qps_current = mean(queries_24h)
        ) %>%
        ungroup() %>%
        mutate(
          qps_current_tile = adam_color_tile(
            qps_current,
            "white", dashboard_colors[3]
          )
        ),
      by = c("Country")
    )

  df_qps_cc_traffic_spark <- df_qps_cc_traffic %>%
    arrange(ts) %>%
    group_by(Country) %>%
    summarise(
      queries_bar = spk_chr(
        queries_24h,
        type = "line",
        lineColor = sparkline_colors[1],
        fillColor = sparkline_colors[2],
        spotColor = TRUE,
        width = 80
      )
    )

  htmltools::attachDependencies(
    htmltools::div(),
    html_dependency_sparkline()
  )

  df_qps_cc_traffic_table %>%
    left_join(df_qps_cc_traffic_spark, by = "Country") %>%
    mutate(
      Country = as.character(Country),
      Country = ifelse(is.na(Country), gettext("unknown"),
        ifelse(Country == gettext("United Kingdom"),
          gettext("UK"),
          ifelse(Country == gettext("United States"),
            gettext("USA"),
            Country
          )
        )
      )
    ) %>%
    arrange(desc(qps_current)) %>%
    select(
      Country,
      qps_max_tile,
      qps_min_tile,
      queries_bar,
      qps_current_tile
    ) %>%
    kable(
      "html",
      table.attr = "class='dtable'",
      escape = FALSE,
      align = c("r", "c", "c", "c", "c"),
      format.args = list(big.mark = " "),
      col.names = c(
        gettext("Country"), "Max QPS", "Min QPS",
        "Trend", gettext("Average")
      )
    ) %>%
    column_spec(1, bold = TRUE, width = "3cm") %>%
    column_spec(2, width = "3cm") %>%
    column_spec(3, width = "3cm") %>%
    column_spec(4, width = "3cm") %>%
    column_spec(5, width = "3cm") %>%
    kable_styling(
      bootstrap_options = c("striped", "hover", "condensed", "responsive"),
      full_width = TRUE,
      position = "left"
    )
}

### Round trip time by qps, month, region and continetn
#' @param df_rtt
#' @param df_qps


plot_rtt_by_continent_region <- function(df_rtt, df_qps) {
  df_rtt_qps <- left_join(df_qps, df_rtt) %>%
    mutate(region = countrycode::countrycode(country_code, "iso2c", "region23"),
      continent = countrycode::countrycode(country_code, "iso2c", "continent"),
      Country = countrycode::countrycode(country_code, "iso2c", "country.name"),
      region = ifelse(region %in% c("Melanesia", "Micronesia", "Polynesia"),
        gettext("Pacific Islands"),
        ifelse(region %in% c("Eastern Africa", "Middle Africa", "Southern Africa"),
          gettext("South-East Africa"),
          ifelse(region %in% c("Eastern Asia", "South-Eastern Asia", "Southern Asia"),
            gettext("South-Eastern Asia"),
            ifelse(region %in% c("Western Asia", "Central Asia"),
              gettext("Western-Central Asia"),
              ifelse(region %in% c("Central America", "Caribbean"),
                gettext("Central America & Caribbean"),
                ifelse(region %in% c("Western Africa", "Northern Africa"),
                  gettext("North-West Africa"), region
                )
              )
            )
          )
        )
      )
    )

  df_rtt_qps_country <- df_rtt_qps %>%
    mutate(ts = as.Date(ts)) %>%
    filter(!is.na(continent)) %>%
    group_by(ts, region, continent) %>%
    summarise(
      total_qps = sum(qps),
      rtt = weighted.mean(mean_median_rtt, qps, na.rm = TRUE)
    ) %>%
    mutate(
      month = month(ts), year = year(ts),
      Date = as.Date(paste(year, month, 1, sep = "-"))
    ) %>%
    group_by(Date, month, year, region, continent) %>%
    summarise(total_qps = round(mean(total_qps), 2), rtt = mean(rtt)) %>%
    mutate(
      continent = ifelse(continent == "Oceania", gettext("Oceania"),
        ifelse(continent == "Americas", gettext("Americas"),
          ifelse(continent == "Europe", gettext("Europe"),
            ifelse(continent == "Asia", gettext("Asia"),
              ifelse(continent == "Africa", gettext("Africa"), continent)
            )
          )
        )
      )
    ) %>%
    filter(!is.na(rtt)) %>%
    ungroup()

  arrow <- df_rtt_qps_country %>%
    filter(month == max(month)) %>%
    mutate(month = as.factor(month))


  rtt_lines <- df_rtt_qps_country %>%
    ggplot(aes(total_qps, rtt,
      color = continent,
      group = region,
      text = paste(
        region, "\n", Date,
        "<b>\nQPS:</b>", total_qps,
        "<b>\nRTT (ms):</b>", round(rtt, 2)
      )
    )) +
    geom_point(size = 0.5) +
    geom_path() +
    geom_point(arrow,
      mapping = aes(total_qps, rtt,
        color = continent,
        group = region, shape = month
      ),
      shape = 1, size = 3
    ) +
    theme_bw() +
    scale_y_continuous(
      labels = function(x) {
        format(x,
          big.mark = " ",
          scientific = FALSE
        )
      }
    ) +
    scale_x_log10() +
    labs(
      x = gettext("Queries per second"),
      y = gettext("Round-trip time")
    ) +
    scale_color_manual(
      values = dashboard_pal(length(unique(df_rtt_qps_country$continent))),
      labels = c(
        gettext("Africa"), gettext("Americas"),
        gettext("Asia"), gettext("Europe"),
        gettext("Oceania")
      )
    )

  rtt_lines <- ggplotly(rtt_lines, tooltip = "text") %>%
    config(displayModeBar = FALSE) %>%
    layout(xaxis = list(fixedrange = TRUE)) %>%
    layout(yaxis = list(fixedrange = TRUE))

  rtt_lines
}


### Timeserie of rtt
#' @param df_rtt
#' @param df_qps

plot_rtt_by_continent <- function(df_rtt, df_qps) {
  df_rtt_qps <- left_join(df_qps, df_rtt) %>%
    mutate(
      region = countrycode::countrycode(country_code, "iso2c", "region23"),
      continent = countrycode::countrycode(country_code, "iso2c", "continent"),
      Country = countrycode::countrycode(country_code, "iso2c", "country.name")
    )

  df_rtt_qps_continent <- df_rtt_qps %>%
    mutate(ts = as.character(as.Date(ts))) %>%
    group_by(ts, continent) %>%
    summarise(
      total_qps = sum(qps),
      rtt = weighted.mean(mean_median_rtt, qps,
        na.rm = TRUE
      )
    ) %>%
    mutate(
      continent = ifelse(continent == "Oceania", gettext("Oceania"),
        ifelse(continent == "Americas", gettext("Americas"),
          ifelse(continent == "Europe", gettext("Europe"),
            ifelse(continent == "Asia", gettext("Asia"),
              ifelse(continent == "Africa", gettext("Africa"), continent)
            )
          )
        )
      )
    ) %>%
    filter(!is.na(continent)) %>%
    ungroup()

  fig <- df_rtt_qps_continent %>%
    plot_ly() %>%
    add_trace(
      x = ~ as.Date(ts),
      y = ~rtt,
      color = ~continent,
      colors = dashboard_colors,
      hovertemplate = ~ paste0(
        continent, "<BR>QPS: <B>",
        round(total_qps, 2), "</B><BR>RTT: <B>",
        round(rtt, 2), "ms</B>"
      ),
      type = "scatter",
      mode = "lines",
      showlegend = TRUE
    ) %>%
    layout(
      xaxis = list(title = gettext("Date")),
      yaxis = list(title = gettext("Round-trip time"))
    )

  fig
}

#' Total traffic Timeserie
#' @param endpoint
#' @param df qps_total
qps_total_plot <- function(df, endpoint) {
  qps_day <- qps_total_1h %>%
    mutate(ts = as.Date(ts)) %>%
    group_by(ts) %>%
    summarise(qps = mean(qps)) %>%
    ungroup()
  plot_ly(
    qps_day,
    x = ~ts,
    y = ~qps,
    name = "DNS",
    type = "scatter",
    mode = "lines",
    line = list(color = dashboard_colors[1]),
    hovertemplate = paste0(
      "<b>", gettext("Date"), "</b>: %{x}",
      "<br><b>qps</b>: %{y}"
    )
  ) %>%
    config(displayModeBar = TRUE, locale = locale_lang_code()) %>%
    onRender(add_csv_button(endpoint)) %>%
    layout(
      xaxis = list(
        title = gettext("Date"),
        rangeselector = list(
          buttons = list(
            list(
              count = 1,
              label = gettext("1 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 3,
              label = gettext("3 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 6,
              label = gettext("6 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              step = "all",
              label = gettext("all")
            )
          ),
          rangeslider = list(type = "date")
        )
      ),
      yaxis = list(title = gettext("Queries per second"))
    )
}
