df_sk1_week <- fromJSONwithToken(api_endpoint("/auctions_auctions_summary")) |>
  select(date, total, auctioned) |>
  mutate(ts = as.Date(date))

df_sk1_week <- df_sk1_week |>
  mutate(
    week = isoweek(ts),
    year = isoyear(ts)
  ) |>
  group_by(year, week) |>
  mutate(
    week_start = floor_date(ts, "week", week_start = 1),
    week_end   = ceiling_date(ts, "week", week_start = 1)
  ) |>
  select(
    ts,
    week_start,
    week_end,
    total,
    auctioned
  ) |>
  reframe(
    total = sum(total),
    auctioned = sum(auctioned),
    perc = round(auctioned * 100 / total, 2),
    week_start = week_start,
    week_end = week_end
  ) |>
  ungroup() |>
  arrange(week_end)

week_plot <- plot_ly(
  df_sk1_week,
  x = ~ as.Date(week_end),
  type = "scatter",
  mode = "lines",
  y = ~total,
  name = gettext("Domains available for auction"),
  line = list(color = dashboard_colors[1]),
  hovertemplate = paste0(
    "<b>", gettext("Domains available for auction"), "</b>: %{y}<extra></extra>"
  )
) |>
  add_trace(
    y = ~auctioned,
    name = gettext("Auctioned domains"),
    mode = "lines",
    line = list(color = dashboard_colors[3]),
    hovertemplate = paste0(
      "<b>", gettext("Auctioned domains"), "</b>: %{y}<extra></extra>"
    )
  ) |>
  add_trace(
    y = 0,
    name = gettext("Date"),
    line = list(color = "rgba(255, 0, 0, 0)"),
    customdata = paste0(
      gettext("from"),
      " <b>",
      format(df_sk1_week$week_start),
      "</b> ",
      gettext("to"),
      " <b>",
      format(df_sk1_week$week_end),
      "</b>"
    ),
    hovertemplate = paste0(
      gettext("Week"), " %{customdata}", "<extra></extra>"
    ),
    visible = TRUE,
    legendrank = 1,
    showlegend = FALSE
  ) |>
  layout(
    hovermode = "x unified",
    margin = list(t = 75),
    title = gettext("Weekly available and auctioned domains"),
    legend = list(
      orientation = "h",
      xanchor = "center",
      x = 0.5,
      y = 1.08
    ),
    separators = ".\u202F",
    xaxis = list(
      title = gettext("Week"),
      rangeselector = list(
        buttons = list(
          list(
            count = 15,
            label = gettext("15 d"),
            step = "day",
            stepmode = "backward"
          ),
          list(
            count = 1,
            label = gettext("1 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            count = 6,
            label = gettext("6 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            count = 1,
            label = gettext("1 y"),
            step = "year",
            stepmode = "backward"
          ),
          list(
            step = "all",
            label = gettext("all")
          )
        ),
        rangeslider = list(type = "date")
      )
    ),
    yaxis = list(
      title = gettext("Number of domains"),
      tickformat = ",d"
    )
  )

ratio <- plot_ly(df_sk1_week,
  x = ~ as.Date(week_end),
  y = ~perc,
  type = "scatter",
  name = paste0(
    gettext("Ratio of auctioned domains"), " (%)"
  ),
  mode = "lines",
  line = list(color = dashboard_colors[2]),
  hovertemplate = paste0(
    "<b>",
    gettext("Domains available for auction"), "</b>: ",
    formatC(df_sk1_week$total, big.mark = " "), "<br><b>",
    gettext("Auctioned domains"), "</b>: ",
    formatC(df_sk1_week$auctioned,
      big.mark = " "
    ), "<br><b>",
    gettext("Ratio (%)"), "</b>: %{y}<br><b><extra></extra>"
  )
) |>
  add_trace(
    y = 0,
    name = gettext("Date"),
    line = list(color = "rgba(255, 0, 0, 0)"),
    customdata = paste0(
      gettext("from"),
      " <b>",
      format(df_sk1_week$week_start),
      "</b> ",
      gettext("to"),
      " <b>",
      format(df_sk1_week$week_end),
      "</b>"
    ),
    hovertemplate = paste0(
      gettext("Week"), " %{customdata}", "<extra></extra>"
    ),
    visible = TRUE,
    legendrank = 1,
    showlegend = FALSE
  ) %>%
  layout(
    xaxis = list(
      title = gettext("Date"),
      rangeselector = list(
        buttons = list(
          list(
            count = 15,
            label = gettext("15 d"),
            step = "day",
            stepmode = "backward"
          ),
          list(
            count = 1,
            label = gettext("1 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            count = 6,
            label = gettext("6 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            count = 1,
            label = gettext("1 y"),
            step = "year",
            stepmode = "backward"
          ),
          list(
            step = "all",
            label = gettext("all")
          )
        ),
        rangeslider = list(type = "date"),
        x = 0.8
      )
    ),
    yaxis = list(
      title = paste0(
        gettext("Ratio of auctioned domains"), " (%)"
      ),
      ticksuffix = "%"
    )
  )



subplot(week_plot, ratio,
  margin = 0.06, titleX = TRUE, titleY = TRUE, nrows = 1
) %>%
  layout(
    hovermode = "x unified",
    autosize = TRUE,
    xaxis = list(showgrid = TRUE, zeroline = FALSE),
    yaxis = list(showgrid = TRUE, zeroline = FALSE)
  ) %>%
  config(displayModeBar = TRUE, locale = locale_lang_code()) |>
  htmlwidgets::onRender(add_csv_button_local())
