df_bidders_country <-
  fromJSONwithToken(api_endpoint("/auctions_bidders_by_country_1w"))

country_codes <- load_country_codes() %>%
  rename("cc" = country_code)

df_bidders_country <- df_bidders_country |>
  mutate(date = as.Date(date)) |>
  rename("cc" = country) |>
  left_join(country_codes, by = c("cc")) |>
  mutate(
    week = isoweek(date),   # ISO week number
    year = isoyear(date)    # ISO year
  ) |>
  group_by(year, week) |>
  mutate(
    week_start = floor_date(date, "week", week_start = 1),
    week_end   = ceiling_date(date, "week", week_start = 1)
  ) |>
  mutate(
    Country = as.factor(Country),
    Country = fct_lump_n(Country,
      n = 10,
      w = total,
      other_level = gettext("Other")
    )
  ) |>
  ungroup() |>
  complete(Country, year, week,
    fill = list(
      total = 0,
      week_start = NA,
      week_end = NA
    )
  ) |>
  group_by(Country, year, week) |>
  mutate(
    total = sum(total),
    week_start = week_start,
    week_end = week_end
  ) |>
  ungroup() |>
  arrange(year, week)

df_cz <- df_bidders_country |>
  left_join(country_codes, by = c("cc")) |>
  filter(cc == "CZ") |>
  select(year, week, week_start, week_end)

df_bidders_country <- df_bidders_country |>
  select(-c(week_start, week_end)) |>
  left_join(df_cz,
    by = c("year", "week"),
    relationship = "many-to-many"
  )

n_countries <- df_bidders_country |>
  select(Country) |>
  unique() |>
  nrow()

df_bidders_country$Country <- factor(df_bidders_country$Country,
  levels = unique(df_bidders_country$Country)[order(df_bidders_country$total, decreasing = TRUE)]
)


plot_ly(
  type = "scatter",
  mode = "lines",
  data = df_bidders_country,
  x = ~week_end,
  y = ~total,
  color = ~Country,
  colors = dashboard_pal(n = n_countries),
  hovertemplate = paste0(
    "<b>", df_bidders_country$Country, "</b>: ", df_bidders_country$total, "<extra></extra>"
  )
) |>
  layout(
    hovermode = "x unified",
    legend = list(
      orientation = "h",
      xanchor     = "center",
      x           = 0.5,
      y           = 1.08
    ),
    xaxis =
      list(
        title = gettext("Week"),
        rangeselector = list(
          buttons = list(
            list(
              count = 15,
              label = gettext("15 d"),
              step = "day",
              stepmode = "backward"
            ),
            list(
              count = 1,
              label = gettext("1 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 6,
              label = gettext("6 m"),
              step = "month",
              stepmode = "backward"
            ),
            list(
              count = 1,
              label = gettext("1 y"),
              step = "year",
              stepmode = "backward"
            ),
            list(
              count = 3,
              label = gettext("3 y"),
              step = "year",
              stepmode = "backward"
            ),
            list(
              count = 6,
              label = gettext("6 y"),
              step = "year",
              stepmode = "backward"
            ),
            list(
              step = "all",
              label = gettext("all")
            )
          ),
          rangeslider = list(type = "date")
        )
      ),
    yaxis = list(title = gettext("Number of bidders"))
  ) |>
  config(
    displayModeBar = TRUE,
    locale = locale_lang_code()
  ) |>
  htmlwidgets::onRender(add_csv_button_local())
