num_bids_time_month <- fromJSONwithToken(api_endpoint("/auctions_bid_times")) |>
  mutate(report_datetime_range = as.character(report_datetime_range)) |>
  separate(
    report_datetime_range,
    into = c("start_time", "end_time"),
    sep = ",",
    extra = "drop"
  ) %>%
  mutate(
    start_time = gsub("\\[|\"", "", start_time),
    end_time = gsub("\\)|\"", "", end_time),
    date = as.Date(date),
    start_time = ymd_hms(start_time, tz = "UTC"),
    cet_cest_start = with_tz(start_time, tzone = "Europe/Prague"),
    end_time = ymd_hms(end_time, tz = "UTC"),
    cet_cest_end = with_tz(end_time, tzone = "Europe/Prague"),
  ) |>
  select(cet_cest_start, count, hour_num) |>
  arrange(cet_cest_start)

df_bids_time_month <- num_bids_time_month |>
  mutate(month = month(cet_cest_start), year = year(cet_cest_start)) |>
  group_by(month, year, hour_num) |>
  mutate(
    month_start = min(cet_cest_start),
    month_end = max(cet_cest_start)
  ) |>
  group_by(month, year, month_start, month_end, hour_num) |>
  summarise(
    mean = mean(count),
    count = sum(count)
  ) |>
  ungroup() |>
  left_join(intervals) |>
  filter(hour_num < 25) |>
  mutate(
    ym = paste0(year, "-", month)
  ) |>
  arrange(year, month, hour_num)

months <- nrow(df_bids_time_month) / 24

df_bids_time_month <- df_bids_time_month |>
  mutate(
    month_order =
      rep(
        1:months,
        each = 24
      )
  )


ticks <- df_bids_time_month %>%
  select(month_end, ym, month_order) %>%
  mutate(
    char = as.character(month_end),
    ym = case_when(
      endsWith(char, "12:00:00") ~ ym,
      TRUE ~ NA_character_
    )
  ) %>%
  drop_na()


full_start_date <- min(df_bids_time_month$month_start)
full_end_date <- max(df_bids_time_month$month_end)

plot_ly(
  df_bids_time_month,
  type = "bar",
  x = ~month_order,
  y = ~mean,
  split = ~hour_num,
  color = ~hour_num,
  colors = dashboard_pal(24),
  hovertemplate =
    ~ paste0(
      ym,
      "<br>", start_time_hour, " – ", end_time_hour, "<br>",
      "<b>", gettext("Mean bids"), "</b>: ",
      round(mean, 2), "<br>",
      "<b>", gettext("Number of bids"), "</b>: ",
      formatC(count, big.mark = " "), "<extra></extra>"
    )
) |>
  layout(
    margin = list(t = 20),
    legend = list(
      title = list(text = gettext("Hour of Day")),
      orientation = "h",
      xanchor = "center",
      x = 0.5,
      y = 1
    ),
    separators = ".\u202F",
    bargap = -0.5,
    xaxis = list(
      title = gettext("Month"),
      range = c(max(df_bids_time_month$month_order) - 11.5,
                max(df_bids_time_month$month_order) + 1),
      tickvals = ~ticks$month_order,
      ticktext = ~ticks$ym,
      tickmode = "array"
    ),
    yaxis = list(
      title = gettext("Mean bids"),
      tickformat = ",d"
    ),
    updatemenus = list(
      list(bgcolor = "#ECECEC",
           borderwidth = 0,
           type = "buttons",
           x = 0.2,
           y = 1.1,
           direction = "right",
           buttons = list(
             list(label = gettext("1 m"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(max(df_bids_time_month$month_order) - 0.5,
                                max(df_bids_time_month$month_order) + 1))),
             list(label = gettext("3 m"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(max(df_bids_time_month$month_order) - 2.5,
                                max(df_bids_time_month$month_order) + 1))),
             list(label = gettext("6 m"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(max(df_bids_time_month$month_order) - 5.5,
                                max(df_bids_time_month$month_order) + 1))),
             list(label = gettext("1 y"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(max(df_bids_time_month$month_order) - 11.5,
                                max(df_bids_time_month$month_order) + 1))),
             list(label = gettext("All"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(min(df_bids_time_month$month_order),
                                max(df_bids_time_month$month_order) + 1)))
           ),
           active = 4) # Set the "All" button as active by default
    )
  ) |>
  config(displayModeBar = TRUE, locale = locale_lang_code()) |>
  htmlwidgets::onRender(add_csv_button_local()) |>
  hide_colorbar()
