df_bids_time_week <- fromJSONwithToken(api_endpoint("/auctions_bid_times")) |>
  mutate(report_datetime_range = as.character(report_datetime_range)) |>
  separate(
    report_datetime_range,
    into = c("start_time", "end_time"),
    sep = ",",
    extra = "drop"
  ) %>%
  mutate(
    start_time = gsub("\\[|\"", "", start_time),
    end_time = gsub("\\)|\"", "", end_time),
    date = as.Date(date),
    start_time = ymd_hms(start_time, tz = "UTC"),
    cet_cest_start = with_tz(start_time, tzone = "Europe/Prague"),
    end_time = ymd_hms(end_time, tz = "UTC"),
    cet_cest_end = with_tz(end_time, tzone = "Europe/Prague"),
  ) |>
  select(cet_cest_start, count, hour_num) |>
  arrange(cet_cest_start)

df_bids_time_week <- df_bids_time_week |>
  mutate(
    week = week(cet_cest_start),
    year = year(cet_cest_start)
  ) |>
  mutate(
    week_start = floor_date(cet_cest_start, unit = "week", week_start = 1),
    week_end   = ceiling_date(cet_cest_start + 1, unit = "week", week_start = 1)
  ) |>
  select(
    cet_cest_start,
    week_start,
    week_end,
    hour_num,
    count
  ) |>
  group_by(week_end, week_start, hour_num) |>
  summarise(
    mean = mean(count),
    count = sum(count)
  ) |>
  ungroup() |>
  arrange(week_end)  |>
  left_join(intervals) |>
  filter(hour_num < 25) |>
  filter(difftime(week_end, week_start) > 0) |>
  mutate(week_start = as.character(week_start))

weeks <- length(unique(df_bids_time_week$week_end))

df_bids_time_week <- df_bids_time_week |>
  mutate(
    week_order =
      rep(
        1:weeks,
        each = 24
      )
  )



ticks <- df_bids_time_week |>
  select(week_end, week_order) |>
  distinct() |>
  mutate(week_end = as.character(week_end - 1),
         week_end = gsub(" .*", "", week_end))

plot_ly(
  df_bids_time_week,
  type = "bar",
  x = ~week_order,
  y = ~mean,
  split = ~hour_num,
  color = ~hour_num,
  colors = dashboard_pal(24),
  hovertemplate =
    ~ paste0(
      gettext("Week"), " ", gettext("from"), " ", as.Date(df_bids_time_week$week_start),
      gettext(" to "), as.Date(df_bids_time_week$week_end),
      "<br>", df_bids_time_week$start_time_hour, " – ",
      df_bids_time_week$end_time_hour,
      "<br><b>", gettext("Mean bids"), "</b>: ",
      round(mean, 2),
      "<br><b>", gettext("Number of bids"), "</b>: ",
      formatC(count, big.mark = " "),
      "<extra></extra>"
    )
) |>
  layout(
    margin = list(t = 20),
    legend = list(
      title = list(text = gettext("Hour of Day")),
      orientation = "h",
      xanchor = "center",
      x = 0.5,
      y = 1
    ),
    separators = ".\u202F",
    xaxis = list(
      title = gettext("Week"),
      range = c(max(df_bids_time_week$week_order) - 3.5,
                max(df_bids_time_week$week_order) + 1), # Default range
      tickvals = ~ticks$week_order,
      ticktext = ~ticks$week_end,
      tickmode = "array"
    ),
    yaxis = list(
      title = gettext("Mean bids"),
      tickformat = ",d"
    ),
    updatemenus = list(
      list(bgcolor = "#ECECEC",
           borderwidth = 0,
           type = "buttons",
           x = 0.2,
           y = 1.1,
           direction = "right",
           buttons = list(
             list(label = gettext("1 w"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(max(df_bids_time_week$week_order - 0.5),
                                max(df_bids_time_week$week_order) + 0.5))),
             list(label = gettext("2 w"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(max(df_bids_time_week$week_order - 1.5),
                                max(df_bids_time_week$week_order) + 0.5))),
             list(label = gettext("1 m"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(max(df_bids_time_week$week_order) - 3.5,
                                max(df_bids_time_week$week_order) + 1))),
             list(label = gettext("3 m"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(max(df_bids_time_week$week_order) - 13.5,
                                max(df_bids_time_week$week_order) + 1))),
             list(label = gettext("6 m"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(max(df_bids_time_week$week_order) - 26.5,
                                max(df_bids_time_week$week_order) + 1))),
             list(label = gettext("1 y"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(max(df_bids_time_week$week_order) - 52.5,
                                max(df_bids_time_week$week_order) + 1))),
             list(label = gettext("All"),
                  method = "relayout",
                  args = list("xaxis.range",
                              c(min(df_bids_time_week$week_order),
                                max(df_bids_time_week$week_order) + 1)))
           ),
           active = 2)
    )
  ) |>
  config(displayModeBar = TRUE, locale = locale_lang_code()) |>
  htmlwidgets::onRender(add_csv_button_local()) |>
  hide_colorbar()
