df_sk3_week <- fromJSONwithToken(api_endpoint("/auctions_auctions_summary")) |>
  select(date, registered, num_paid) |>
  mutate(
    ts = as.Date(date),
    num_paid = replace_na(num_paid, 0),
    registered = replace_na(registered, 0)
  )

df_sk3_week <- df_sk3_week |>
  mutate(
    week = isoweek(ts),
    year = isoyear(ts)
  ) |>
  group_by(year, week) |>
  mutate(
    week_start = floor_date(ts, "week", week_start = 1),
    week_end   = ceiling_date(ts, "week", week_start = 1)
  ) |>
  select(
    ts,
    week_start,
    week_end,
    registered,
    num_paid
  ) |>
  reframe(
    registered = sum(registered),
    num_paid = sum(num_paid),
    perc = round(registered * 100 / num_paid, 2),
    week_start = week_start,
    week_end = week_end
  ) |>
  ungroup() |>
  arrange(week_end)


week_plot <- plot_ly(
  df_sk3_week,
  x = ~ as.Date(week_end),
  type = "scatter",
  mode = "lines",
  y = ~num_paid,
  name = gettext("Number paid domains"),
  mode = "lines",
  line = list(color = dashboard_colors[1]),
  hovertemplate = paste0(
    "<b>", gettext("Number of paid domains"), "</b>: %{y}<extra></extra>"
  )
) |>
  add_trace(
    y = ~registered,
    name = gettext("Registered domains"),
    line = list(color = dashboard_colors[3]),
    hovertemplate = paste0(
      "<b>", gettext("Registered domains"), "</b>: %{y}<extra></extra>"
    )
  ) |>
  add_trace(
    y = 0,
    name = gettext("Date"),
    line = list(color = "rgba(255, 0, 0, 0)"),
    customdata = paste0(
      gettext("from"),
      " <b>",
      format(df_sk3_week$week_start),
      "</b> ",
      gettext("to"),
      " <b>",
      format(df_sk3_week$week_end),
      "</b>"
    ),
    hovertemplate = paste0(
      gettext("Week"), " %{customdata}", "<extra></extra>"
    ),
    visible = TRUE,
    legendrank = 1,
    showlegend = FALSE
  ) |>
  config(displayModeBar = TRUE, locale = locale_lang_code()) |>
  htmlwidgets::onRender(add_csv_button_local()) |>
  layout(
    hovermode = "x unified",
    margin = list(t = 75),
    title = gettext("Weekly registered domains"),
    legend = list(
      orientation = "h",
      xanchor = "center",
      x = 0.5,
      y = 1.08
    ),
    separators = ".\u202F",
    xaxis = list(
      title = gettext("Week"),
      rangeselector = list(
        buttons = list(
          list(
            count = 15,
            label = gettext("15 d"),
            step = "day",
            stepmode = "backward"
          ),
          list(
            count = 1,
            label = gettext("1 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            count = 6,
            label = gettext("6 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            count = 1,
            label = gettext("1 y"),
            step = "year",
            stepmode = "backward"
          ),
          list(
            step = "all",
            label = gettext("all")
          )
        ),
        rangeslider = list(type = "date")
      )
    ),
    yaxis = list(
      title = gettext("Number of domains"),
      tickformat = ",d"
    )
  )

ratio <- plot_ly(df_sk3_week,
  x = ~ as.Date(week_end),
  y = ~perc,
  type = "scatter",
  name = paste0(
    gettext("Ratio of registered domains"), " (%)"
  ),
  mode = "lines",
  line = list(color = dashboard_colors[2]),
  hovertemplate = paste0(
    "<b>",
    gettext("Number of paid domains"), "</b>: ",
    formatC(df_sk3_week$num_paid,
      big.mark = " "
    ),
    "<br><b>",
    gettext("Registered domains"), "</b>: ",
    formatC(df_sk3_week$registered,
      big.mark = " "
    ), "<br><b>",
    gettext("Ratio (%)"), "</b>: %{y}<br><b><extra></extra>"
  )
) |>
  add_trace(
    y = 0,
    name = gettext("Date"),
    line = list(color = "rgba(255, 0, 0, 0)"),
    customdata = paste0(
      gettext("from"),
      " <b>",
      format(df_sk3_week$week_start),
      "</b> ",
      gettext("to"),
      " <b>",
      format(df_sk3_week$week_end),
      "</b>"
    ),
    hovertemplate = paste0(
      gettext("Week"), " %{customdata}", "<extra></extra>"
    ),
    visible = TRUE,
    legendrank = 1,
    showlegend = FALSE
  ) %>%
  layout(
    xaxis = list(
      title = gettext("Date"),
      rangeselector = list(
        buttons = list(
          list(
            count = 15,
            label = gettext("15 d"),
            step = "day",
            stepmode = "backward"
          ),
          list(
            count = 1,
            label = gettext("1 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            count = 6,
            label = gettext("6 m"),
            step = "month",
            stepmode = "backward"
          ),
          list(
            count = 1,
            label = gettext("1 y"),
            step = "year",
            stepmode = "backward"
          ),
          list(
            step = "all",
            label = gettext("all")
          )
        ),
        rangeslider = list(type = "date"),
        x = 0.8
      )
    ),
    yaxis = list(
      title = paste0(
        gettext("Ratio of registered domains"), " (%)"
      ),
      ticksuffix = "%"
    )
  )

subplot(week_plot, ratio,
  margin = 0.06, titleX = TRUE, titleY = TRUE, nrows = 1
) %>%
  layout(
    hovermode = "x unified",
    autosize = TRUE,
    xaxis = list(showgrid = TRUE, zeroline = FALSE),
    yaxis = list(showgrid = TRUE, zeroline = FALSE)
  ) %>%
  config(displayModeBar = TRUE, locale = locale_lang_code()) |>
  htmlwidgets::onRender(add_csv_button_local())
