library(flexdashboard)
library(dplyr)
library(gridExtra)
library(ggplot2)
library(plotly)
library(lubridate)
library(tidyr)
library(knitr)
library(kableExtra)
library(formattable)
library(sparkline)
library(jsonlite)
library(forcats)
library(colorspace)
library(stringi)

root_endpoint <- "https://stats.nic.cz"
locale_domain <- "R-Dashboard"

bindtextdomain(locale_domain, "./locales")

#' Try to use the translation of text according to locale setting.
#'
#' This function overrides the same function from base package with a fixed
#' domain parameter.
gettext <- function(text) {
  base::gettext(text, domain = locale_domain)
}

#' Return URL of a REST API endpoint
#'
#' @param endpoint Name of the endpoint, as defined in OpenAPI (starting
#'   with a slash).
#' @param ... Any number of named parameters that are appended as URL
#'   parameters.
#' @return A complete URL of the endpoint, with optional parameters.
#' @examples
#' api_endpoint("/cz_qps_server_1h", server = "eq.manitu", limit = 10)
api_endpoint <- function(endpoint, ...) {
  pars <- list(...)
  if (length(pars) == 0) {
    return(paste0(root_endpoint, endpoint))
  }
  pstr <- mapply(function(x, y) paste(x, URLencode(toString(y)), sep = "="),
                 names(pars), pars)
  paste0(root_endpoint, endpoint, "?", paste(pstr, collapse = "&"))
}

#' Return a JS function used to add an attribute
#' which will be picked up by a code which adds
#' CSV download buttons to Plotly charts.
#'
#' Run in `htmlwidgets::onRender` on Plotly chart objects.
#'
#' @param endpoint Endpoint URL as returned by api_endpoint()
#' @param series DB column name to filter by (optional)
#' @param mapping Dataframe mapping names of chart series to
#'                values in DB (optional, requires series to
#'                be passed as well)
#' @return JS function code
#' @examples
#' … %>% onRender(add_csv_button("/cz_qps_ipv_1h")) %>% …
#'
#' %>% onRender(
#'   add_csv_button(
#'     api_endpoint("/cz_qps_ipv_1h"),
#'     series = "ipv",
#'     mapping = csv_mapping_ipv
#'     )
#' ) %>%

add_csv_button <- function(endpoint = NA, series, mapping) {
  if (is.na(endpoint)) {
    return
  }
  filter_series <- FALSE
  if (!(missing(series) || missing(mapping))) {
    filter_series <- TRUE
    colnames(mapping) <- NULL
  }
  return(paste0(
    "function(plot) {",
    "plot.setAttribute('data-csv-endpoint', '", endpoint, "');",
    ifelse(filter_series, paste0(
      "plot.setAttribute('data-csv-series', '", series, "');",
      "plot.setAttribute('data-csv-mapping', '",
      gsub("\n", "", jsonlite::base64_enc(
        stri_escape_unicode(
          jsonlite::toJSON(mapping)
        )
      )), "');"
    ), ""),
    "}"
  ))
}

#' Returns a JS function used to add an attribute
#' which will be picked up by a code which adds
#' CSV download buttons to Plotly charts. This is
#' for the locally generated version (works with
#' line and bar charts, not Sankey etc.)
#'
#' Run in `htmlwidgets::onRender` on Plotly chart objects.
#'
#' @return JS function code
#' @examples
#'
#' %>% onRender(
#'   add_csv_button_local()
#' ) %>%

add_csv_button_local <- function() {
  return(paste0(
    "function(plot) {",
    "plot.setAttribute('data-csv-local', 'true');",
    "}"
  ))
}

#' Returns a JS function used to mark the chart
#' animation to be set to the last frame after page load.
#'
#' Run in `htmlwidgets::onRender` on Plotly chart objects.
#'
#' @return JS function code
#' @examples
#'
#' %>% onRender(
#'   anim_skip_to_last_frame()
#' ) %>%

anim_skip_to_last_frame <- function() {
  return(paste0(
    "function(plot) {",
    "plot.setAttribute('data-anim-skip-to-last', 'true');",
    "}"
  ))
}

# Mappings for commonly used  chart series to use with add_csv_button:
csv_mapping_ipv <- data.frame("plotly" = c("IPv4", "IPv6"), "db" = c(4, 6))
csv_mapping_prot <- data.frame("plotly" = c("TCP", "UDP"), "db" = c(6, 17))

#' `color_tile` from `formattable` tweaked to use white
#' text when the background is dark for better contrast.
color_tile2 <- function(...,
                        color_low = "white",
                        color_high = "black",
                        color_ratio = 0.85) {
  formatter(
    "span",
    style = function(x) {
      style(
        display = "block",
        padding = "0 4px",
        `border-radius` = "4px",
        `background-color` = csscolor(gradient(as.numeric(x), ...)),
        color = ifelse(x / max(x) > color_ratio, color_low, color_high)
      )
    }
  )
}

#' Wrapper for `color_tile2` with common parameters for number formatting.
adam_color_tile <- function(value, color1, color2) {
  color_tile2(color1, color2)(
    accounting(
      value,
      big.mark = " ",
      decimal.mark = ",",
      format = "d",
      digits = 0
    )
  )
}

#' Wrapper for `color_tile2` with custom decimal places formatting.
adam_color_tile2 <- function(value, color1, color2, decimals) {
  color_tile2(color1, color2)(
    accounting(
      value,
      big.mark = " ",
      decimal.mark = ",",
      format = "f",
      digits = decimals
    )
  )
}

#' Creates a <details> HTML element with summary and
#' toggleable content.
#'
#' @param summary
#' @param content
#' @param tooltip
#' @return HTML element from htmltools
details <- function(summary, content, tooltip = gettext("more info")) {
  htmltools::HTML(paste0(
    "<details title='",
    tooltip, "'><summary><span>",
    summary, "</span></summary>",
    "<div class='details-content'>",
    content,
    "</div></details>"
  ))
}


#' Returns a date formatting string for current locale
#' Defaults to English for unsupported locales
locale_date_format <- function() {
  ifelse(grepl("^cs_CZ", Sys.getlocale("LC_TIME")), "%d.%m.%Y", "%B %-d, %Y")
}


#' Returns a 2-letter code from current locale (en_US.UTF8 -> en)
#' Defaults to English for unsupported locales
locale_lang_code <- function() {
  ifelse(grepl("^cs_CZ", Sys.getlocale("LC_TIME")), "cs", "en")
}

#' Loads country CSV for current locale
#' Defaults to English for unsupported locales
load_country_codes <- function() {
  country_codes <- read.csv(
    paste0("../../Dashboard/CSV/country_codes_", locale_lang_code(), ".csv"),
    sep = ";"
  )
  return(country_codes)
}

#' Wrapper for flexdashboard::valueBox which autoupdates
#' the data from API using JS.
#'
#' It uses the first array element from JSON, so use it
#' with the `*_latest` endpoints or add something like
#' `limit=1, order=ts.desc` to endpoint parameters.
#'
#' @param autoupdate_url Endpoint URL to update from (use endpoint(…))
#' @param autoupdate_field JSON key name containing the value
#' @param autoupdate_interval Polling interval in seconds (if not set or zero,
#'                            the box is updated after page load only)
#' @param autoupdate_separator thousands separator for integer values (default
#'                             is a non-breaking space), non-integer values are
#'                             just printed as-is
#' @param autoupdate_min_value Minimum value that should be shown in the
#'                             dashboard (used to prevent having negative
#'                             numbers in boxes labelled as "increases"
#'                             since midnight)
valueBoxAutoupdated <- function(value, # nolint because lintr wants snake_case
                                autoupdate_url = NULL,
                                autoupdate_field = NULL,
                                autoupdate_interval = NULL,
                                autoupdate_separator = " ",
                                autoupdate_min_value = NULL,
                                caption = NULL,
                                icon = NULL,
                                color = NULL,
                                href = NULL) {
  caption <- tags$div(
    `data-autoupdate-url` = autoupdate_url,
    `data-autoupdate-field` = autoupdate_field,
    `data-autoupdate-interval` = autoupdate_interval,
    `data-autoupdate-separator` = autoupdate_separator,
    `data-autoupdate-min-value` = autoupdate_min_value,
    caption
  )
  return(flexdashboard::valueBox(value, caption, icon, color, href))
}

#' Creates a timestamp filter string to be used with Postgrest API.
#'
#' @param unit
#' @param n
#' @param operator
get_ts <- function(unit = days, n = 30, operator = "gte") {
  sprintf("%s.%s", operator, Sys.Date() %m-% unit(n))
}

#' Color palettes
dashboard_colors <- c("#003893", "#3DB7E4", "#F4587A")
dashboard_colors_darker <- darken(dashboard_colors,
                                  amount = 0.3)
dashboard_colors_lighter <- desaturate(lighten(dashboard_colors,
                                               amount = 0.6),
                                       amount = 0.6)
sparkline_colors <- c(
  desaturate(dashboard_colors[1],
    amount = 0.8
  ),
  desaturate(lighten(dashboard_colors[1],
               amount = 0.8
             ), amount = 1)
)
dashboard_pal <- colorRampPalette(dashboard_colors)

#' Loads country flag emoji from a CSV file
load_country_flags <- function() {
  read.csv("../../Dashboard/CSV/country_flags.csv", sep = ";")
}

#' Creates HTML elements for displaying a country flag.
#'
#' @param code
#' @param flag
get_country_flag_html <- function(code, flag) {
  if (any(is.na(code)) || any(code == "other")) {
    return()
  }
  return(paste0(
    "<span class=flag title=",
    code,
    ">",
    flag,
    "</span> ",
    "<span class=hidden>",
    code,
    "</span>"
  ))
}
